#!/usr/bin/env sh
# SPDX-License-Identifier: AGPL-3.0-or-later
# SPDX-FileCopyrightText: 2021-2024 JayVii <jayvii[AT]posteo[DOT]de>

# $1 - HTML Root
# $2 - prv/pub
# $3 - repo name

# Error out if not all options are given
if [ -z "$3" ]; then
    echo "Usage:
git_update_repo_html.sh \\
    \"/var/www/git.example.git\" \\
    \"pub\" \\
    \"stagit-scripts\"
    "
    exit 1;
fi

# HTML Viewer
cd "${1}/${2}/${3}" || exit 1;

# Cleanup directory
find "${1}/${2}/${3}" \
    -type f \
    -name "*.html" \
    -not -path "${1}/${2}/${3}/exports/*" \
    -exec rm {} \;

# build HTML
stagit "${1}/${2}/${3}.git"

# Modules ----------------------------------------------------------------------

# Activity module
if [ -f "${1}/bin/activity.sh" ]; then
    ACTIVITY_HTML=`${1}/bin/activity.sh "${1}/${2}/${3}.git"`
fi

# Summary Page -----------------------------------------------------------------

# remove pre-existing index
if [ -f "${1}/${2}/${3}/index.html" ]; then
    rm "${1}/${2}/${3}/index.html"
fi

## fetch repo description, URL and menu-line
RDESC=$(
    cat "${1}/${2}/${3}.git/description" 2>/dev/null | \
    sed -e 's/\//\\\//g'
)
RURL=$(cat "${1}/${2}/${3}.git/url" 2>/dev/null)
RURL_ESC=$(echo "$RURL" | sed -e 's/\//\\\//g')
RMENU=$(
    grep -E "^.*<a\ href=\"log.html\">.*$" \
    --only-matching "${1}/${2}/${3}/log.html" | \
    sed -e 's/\//\\\//g'
)

## read template sections
THEAD=$(
    cat "${1}/templates/head.html" | \
    sed \
        -e "s/<\!--TITLE-->/${3}/g" \
        -e "s/<\!--DESC-->/${RDESC}/g" \
        -e "s/<\!--DIR-->/\./g" \
        -e "s/<\!--REPO-->/${RURL_ESC}/g"
)
TMENU=$(
    cat "${1}/templates/menu.html" | \
    sed \
        -e "s/<\!--TITLE-->/${3}/g" \
        -e "s/<\!--DESC-->/${RDESC}/g" \
        -e "s/<\!--REPO-->/${RURL_ESC}/g" \
        -e "s/<\!--MENU-->/${RMENU}/g"
)

## re-format README to summary description
# find readme
READMEFILE=$(
    find "${1}/${2}/${3}/file" -type f -name "*README*" | \
    head -n 1
)
# if readme exists, re-format to summary description
if [ ! -z "$READMEFILE" ]; then
    RSUMD=$(
        grep -E "<a\ href=\"#l[0-9]+\"\ class=\"line\"\ id=\"l[0-9]+\">" \
        "$READMEFILE" | \
        sed -E \
            -e 's/^<a\ .*?<\/a>\s*//g' \
            -e 's/&nbsp;/ /g' \
            -e ' s/&amp;/\&/g' \
            -e 's/&lt;/\</g' \
            -e 's/&gt;/\>/g' \
            -e 's/&quot;/\"/g' \
            -e 's/&#39;/\'"'"'/g' \
            -e 's/&ldquo;/\"/g' \
            -e 's/&rdquo;/\"/g' | \
        pandoc --to=html
    )
fi

## Write template
echo "<!DOCTYPE html>
<html>
${THEAD}
<body>
${TMENU}
<div id=\"content\">
${ACTIVITY_HTML}
<div id=\"description\">
${RSUMD}
</div>
<hr>
<span>Clone this repository via:</span><br>
<span>git clone <a href=\"$RURL\">$RURL</a></span>
</a>
</div>
</body>
</html>
" > "${1}/${2}/${3}/index.html"

# Exports Page -----------------------------------------------------------------

## fetch repo description, URL and menu-line
RDESC=$(
    cat "${1}/${2}/${3}.git/description" 2>/dev/null | \
    sed -e 's/\//\\\//g'
)
RURL=$(
    cat "${1}/${2}/${3}.git/url" 2>/dev/null | \
    sed -e 's/\//\\\//g')
RMENU=$(
    grep -E "^.*<a\ href=\"log.html\">.*$" \
    --only-matching "${1}/${2}/${3}/log.html" | \
    sed -e 's/\//\\\//g'
)

# read template sections
THEAD=$(
    cat "${1}/templates/head.html" | \
    sed \
        -e "s/<\!--TITLE-->/Exports\ -\ ${3}/g" \
        -e "s/<\!--DESC-->/${RDESC}/g" \
        -e "s/<\!--DIR-->/\./g" \
        -e "s/<\!--REPO-->/${RURL}/g"
)
TMENU=$(
    cat "${1}/templates/menu.html" | \
    sed \
        -e "s/<\!--TITLE-->/${3}/g" \
        -e "s/<\!--DESC-->/${RDESC}/g" \
        -e "s/<\!--REPO-->/${RURL}/g" \
        -e "s/<\!--MENU-->/${RMENU}/g"
)

# list exports
EXPTS=$(
    ls "${1}/${2}/${3}/exports/" -lht \
        --time-style="+%Y-%m-%d %H:%m" | \
    awk '{ print "<tr><td><a href=\"exports/" $8 "\">" $8 "</a></td><td>" $6 " " $7 "</td><td>" $5 "</td><tr>" }'
)

## Write template
echo "<!DOCTYPE html>
<html>
${THEAD}
<body>
${TMENU}
<div id=\"content\">
<table id=\"exports\">
<thead>
<tr><td><b>Name</b></td><td><b>Date</b></td><td class=\"num\" align=\"right\"><b>Size</b></td></tr>
</thead>
<tbody>
${EXPTS}
</tbody>
</table>
</div>
</body>
</html>
" > "${1}/${2}/${3}/exports.html"

# Replacements -----------------------------------------------------------------

# Protect Mail-Adresses in HTML
MAILEXP='s/([a-zA-Z0-9\.]+)\@([a-zA-Z0-9\-]+)\.([a-z]+)/\1\[AT\]\2\[DOT\]\3/g'
find "${1}/${2}/${3}/" -type f -exec sed -E -e ${MAILEXP} -i {} \;

# Add Exports to menu (behind Files)
INJEXP="s/(>Files<\/a>\ \|)/\1 <a href=\"\/${2}\/${3}\/exports.html\">Exports<\/a> \|/"
find "${1}/${2}/${3}/" -type f -name "*.html" -exec sed -E "${INJEXP}" -i {} \;

# Add Home to menu (first item)
INJEXP="s/(^.*log.html\">.*$)/<a href=\"\/${2}\/${3}\/index.html\">Home<\/a>\ \|\ \1/"
find "${1}/${2}/${3}/" -type f -name "*.html" -exec sed -E "${INJEXP}" -i {} \;

# Add RSS Feed to menu (last item)
INJEXP="s/(^.*log.html\">.*)(<\/td>)/\1\ \|\ <a href=\"\/${2}\/${3}\/atom.xml\">RSS<\/a>\2/"
find "${1}/${2}/${3}/" -type f -name "*.html" -exec sed -E "${INJEXP}" -i {} \;

# Add Back-Button and Summary page (within title)
INJEXP="s/(<h1>)(${3})(<\/h1>)/\1<a\ href=\"\/${2}\/\">${2}<\/a>\ \/\ <a\ href=\"\/${2}\/${3}\/index.html\">\2<\/a>\3/"
find "${1}/${2}/${3}/" -type f -name "*.html" -exec sed -E "${INJEXP}" -i {} \;

# Replace "git clone" href
INJEXP="s/(git\ clone)\ <a href=\"(https:\/\/.*?\.git)\">/<a href='javascript:navigator.clipboard.writeText(\"\1 \2\");alert(\"Copied to your clipboard:\\\n\1 \2\");'>\1 /g"
find "${1}/${2}/${3}/" -type f -name "*.html" -exec sed -E "${INJEXP}" -i {} \;

# Inject "mark as active" javascript at the end of each html file
INJEXP="s/(<\/html>)/<script\ async\ src=\"\/assets\/js\/mark_as_active.js\"><\/script>\1/"
find "${1}/${2}/${3}/" -type f -name "*.html" -exec sed -E "${INJEXP}" -i {} \;

# Linking Assets ---------------------------------------------------------------
ln -sf "${1}/assets/style.css" "${1}/${2}/${3}/style.css"
ln -sf "${1}/assets/favicon.png" "${1}/${2}/${3}/favicon.png"
ln -sf "${1}/assets/backicon.png" "${1}/${2}/${3}/logo.png"

